# -*- coding: UTF-8 -*-
#
# Copyright (c) 2019-2025   Beijing Tingyu Technology Co., Ltd.
# Copyright (c) 2025        Lybic Development Team <team@lybic.ai, lybic@tingyutech.com>
# Copyright (c) 2025        Lu Yicheng <luyicheng@tingyutech.com>
#
# Author: AEnjoy <aenjoyable@163.com>
#
# These Terms of Service ("Terms") set forth the rules governing your access to and use of the website lybic.ai
# ("Website"), our web applications, and other services (collectively, the "Services") provided by Beijing Tingyu
# Technology Co., Ltd. ("Company," "we," "us," or "our"), a company registered in Haidian District, Beijing. Any
# breach of these Terms may result in the suspension or termination of your access to the Services.
# By accessing and using the Services and/or the Website, you represent that you are at least 18 years old,
# acknowledge that you have read and understood these Terms, and agree to be bound by them. By using or accessing
# the Services and/or the Website, you further represent and warrant that you have the legal capacity and authority
# to agree to these Terms, whether as an individual or on behalf of a company. If you do not agree to all of these
# Terms, do not access or use the Website or Services.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

"""mcp.py: MCP client for lybic MCP(Model Context Protocol) and Restful Interface API."""
import asyncio
from typing import overload

import httpx
from mcp import ClientSession
from mcp.client.streamable_http import streamablehttp_client
from mcp.types import CallToolResult

from lybic import dto
from lybic._api import deprecated
from lybic.lybic import LybicClient


class MCP:
    """MCP is an async client for lybic MCP(Model Context Protocol) and Restful Interface API."""
    def __init__(self, client: LybicClient):
        """
        Init MCP client with lybic client

        :param client: LybicClient
        """
        self.client = client

    async def list(self) -> dto.ListMcpServerResponse:
        """
        List all MCP servers in the organization

        :return:
        """
        self.client.logger.debug("List MCP servers request")
        response = await self.client.request(
            "GET",
            f"/api/orgs/{self.client.org_id}/mcp-servers")
        self.client.logger.debug(f"List MCP servers response: {response.text}")
        return dto.ListMcpServerResponse.model_validate_json(response.text)

    @overload
    async def create(self, data: dto.CreateMcpServerDto) -> dto.McpServerResponseDto: ...

    @overload
    async def create(self, **kwargs) -> dto.McpServerResponseDto: ...

    async def create(self, *args, **kwargs) -> dto.McpServerResponseDto:
        """
        Create a mcp server

        :param data:
        :return:
        """
        if args and isinstance(args[0], dto.CreateMcpServerDto):
            data = args[0]
        elif "data" in kwargs and isinstance(kwargs["data"], dto.CreateMcpServerDto):
            data = kwargs["data"]
        else:
            data = dto.CreateMcpServerDto(**kwargs)
        self.client.logger.debug(f"Create MCP server request: {data.model_dump_json()}")
        response = await self.client.request(
            "POST",
            f"/api/orgs/{self.client.org_id}/mcp-servers",
            json=data.model_dump())
        self.client.logger.debug(f"Create MCP server response: {response.text}")
        return dto.McpServerResponseDto.model_validate_json(response.text)

    async def get_default(self) -> dto.McpServerResponseDto:
        """
        Get default mcp server

        :return:
        """
        self.client.logger.debug("Get default MCP server request")
        response = await self.client.request(
            "GET",
            f"/api/orgs/{self.client.org_id}/mcp-servers/default")
        self.client.logger.debug(f"Get default MCP server response: {response.text}")
        return dto.McpServerResponseDto.model_validate_json(response.text)

    async def delete(self, mcp_server_id: str) -> None:
        """
        Delete a mcp server

        :param mcp_server_id:
        :return:
        """
        self.client.logger.debug(f"Delete MCP server request: {mcp_server_id}")
        await self.client.request("DELETE", f"/api/orgs/{self.client.org_id}/mcp-servers/{mcp_server_id}")

    async def set_sandbox(self, mcp_server_id: str, sandbox_id: str) -> None:
        """
        Set MCP server to a specific sandbox

        :param mcp_server_id: The ID of the MCP server
        :param sandbox_id: The ID of the sandbox to connect the MCP server to
        :return: None
        """
        data = dto.SetMcpServerToSandboxResponseDto(sandboxId=sandbox_id)
        self.client.logger.debug(f"Set MCP server to sandbox request: {data.model_dump_json()}")
        await self.client.request(
            "POST",
            f"/api/orgs/{self.client.org_id}/mcp-servers/{mcp_server_id}/sandbox",
            json=data.model_dump())

    async def call_tool_async(self,
                              mcp_server_id: str,
                              tool_name: str = "computer-use",
                              tool_args: dict = None) -> CallToolResult:
        """
        Call a tool on mcp server

        :param mcp_server_id:
        :param tool_name:
        :param tool_args:
        :return:
        """
        self.client.logger.debug(f"Call tool request: {tool_name} with arguments: {tool_args}")

        last_exception = None
        for attempt in range(self.client.max_retries + 1):
            try:
                async with streamablehttp_client(self.client.make_mcp_endpoint(mcp_server_id),
                                                 headers=self.client.headers,
                                                 timeout=self.client.timeout
                ) as (
                        read_stream,
                        write_stream,
                        _,
                ):
                    async with ClientSession(read_stream, write_stream) as session:
                        await session.initialize()
                        result = await session.call_tool(tool_name, tool_args)
                        self.client.logger.debug(f"Call tool response: {result.model_dump_json()}")
                        return result

            except (httpx.RequestError, httpx.HTTPStatusError) as e:
                last_exception = e
                if attempt < self.client.max_retries:
                    self.client.logger.debug(f"Call tool failed (attempt {attempt + 1}/{self.client.max_retries + 1}): {str(e)}")
                    await asyncio.sleep(2 ** attempt)
                else:
                    self.client.logger.error(f"Call tool failed after {self.client.max_retries + 1} attempts")

        raise RuntimeError(f"Failed to call tool: {last_exception}") from last_exception

class ComputerUse:
    """ComputerUse is an async client for lybic ComputerUse API(MCP and Restful)."""
    def __init__(self, client: LybicClient):
        self.client = client
    @deprecated(
        since="0.7.0",
        removal="1.0.0",
        message="Use parse_llm_output instead"
    )
    @overload
    async def parse_model_output(self, data: dto.ComputerUseParseRequestDto) -> dto.ComputerUseActionResponseDto: ...
    @deprecated(
        since="0.7.0",
        removal="1.0.0",
        message="Use parse_llm_output instead"
    )
    @overload
    async def parse_model_output(self, **kwargs) -> dto.ComputerUseActionResponseDto: ...
    @deprecated(
        since="0.7.0",
        removal="1.0.0",
        message="Use parse_llm_output instead"
    )
    async def parse_model_output(self, *args, **kwargs) -> dto.ComputerUseActionResponseDto:
        """
        parse doubao-ui-tars output

        :param data:
        :return:
        """
        if args and isinstance(args[0], dto.ComputerUseParseRequestDto):
            data = args[0]
        elif "data" in kwargs and isinstance(kwargs["data"], dto.ComputerUseParseRequestDto):
            data = kwargs["data"]
        else:
            data = dto.ComputerUseParseRequestDto(**kwargs)
        self.client.logger.debug(f"Parse model output request: {data.model_dump_json()}")
        response = await self.client.request(
            "POST",
            "/api/computer-use/parse",
            json=data.model_dump(exclude_none=True))
        self.client.logger.debug(f"Parse model output response: {response.text}")
        return dto.ComputerUseActionResponseDto.model_validate_json(response.text)
    async def parse_llm_output(
        self, model_type: dto.ModelType | str, llm_output: str
    ) -> dto.ComputerUseActionResponseDto:
        """Parse LLM output to computer use actions.

        Args:
            model_type: The type of the large language model.
            llm_output: The text output from the large language model.

        Returns:
            A DTO containing the parsed computer use actions.
        """
        if isinstance(model_type, dto.ModelType):
            model = model_type.value
        elif isinstance(model_type, str):
            valid_models = [item.value for item in dto.ModelType]
            if model_type not in valid_models:
                raise ValueError(f"Invalid model_type: {model_type}. Must be one of {valid_models}")
            model = model_type
        else:
            raise TypeError("model_type must be either dto.ModelType or str")

        response = await self.client.request(
            "POST",
            f"/api/computer-use/parse/{model}",
            json={"textContent": llm_output},
        )
        self.client.logger.debug(f"Parse model output response: {response.text}")
        return dto.ComputerUseActionResponseDto.model_validate_json(response.text)

    @overload
    async def execute_computer_use_action(self, sandbox_id: str,
                                    data: dto.ComputerUseActionDto) -> dto.SandboxActionResponseDto: ...
    @overload
    async def execute_computer_use_action(self, sandbox_id: str, **kwargs) -> dto.SandboxActionResponseDto: ...

    async def execute_computer_use_action(self, sandbox_id: str, *args, **kwargs) -> dto.SandboxActionResponseDto:
        """
        Execute a computer use action

        is same as sandbox.Sandbox.execute_computer_use_action
        """
        if args and isinstance(args[0], dto.ComputerUseActionDto):
            data = args[0]
        elif "data" in kwargs:
            data_arg = kwargs["data"]
            if isinstance(data_arg, dto.ComputerUseActionDto):
                data = data_arg
            elif isinstance(data_arg, dict):
                data = dto.ComputerUseActionDto(**data_arg)
            else:
                raise TypeError(f"The 'data' argument must be of type {dto.ComputerUseActionDto.__name__} or dict")
        else:
            data = dto.ComputerUseActionDto(**kwargs)
        self.client.logger.debug(f"Execute computer use action request: {data.model_dump_json()}")
        response = await self.client.request("POST",
                                       f"/api/orgs/{self.client.org_id}/sandboxes/{sandbox_id}/actions/computer-use",
                                       json=data.model_dump(exclude_none=True))
        self.client.logger.debug(f"Execute computer use action response: {response.text}")
        return dto.SandboxActionResponseDto.model_validate_json(response.text)



# -*- coding: UTF-8 -*-
#
# Copyright (c) 2019-2025   Beijing Tingyu Technology Co., Ltd.
# Copyright (c) 2025        Lybic Development Team <team@lybic.ai, lybic@tingyutech.com>
# Copyright (c) 2025        Lu Yicheng <luyicheng@tingyutech.com>
#
# Author: AEnjoy <aenjoyable@163.com>
#
# These Terms of Service ("Terms") set forth the rules governing your access to and use of the website lybic.ai
# ("Website"), our web applications, and other services (collectively, the "Services") provided by Beijing Tingyu
# Technology Co., Ltd. ("Company," "we," "us," or "our"), a company registered in Haidian District, Beijing. Any
# breach of these Terms may result in the suspension or termination of your access to the Services.
# By accessing and using the Services and/or the Website, you represent that you are at least 18 years old,
# acknowledge that you have read and understood these Terms, and agree to be bound by them. By using or accessing
# the Services and/or the Website, you further represent and warrant that you have the legal capacity and authority
# to agree to these Terms, whether as an individual or on behalf of a company. If you do not agree to all of these
# Terms, do not access or use the Website or Services.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

"""lybic.py is the main entry point for Lybic API."""
import asyncio
import os
import httpx

from lybic.base import _LybicBaseClient


class LybicClient(_LybicBaseClient):
    """LybicAsyncClient is a client for all Lybic API."""

    def __init__(self,
                 org_id: str = os.getenv("LYBIC_ORG_ID"),
                 api_key: str = os.getenv("LYBIC_API_KEY"),
                 endpoint: str = os.getenv("LYBIC_API_ENDPOINT", "https://api.lybic.cn"),
                 timeout: int = 10,
                 extra_headers: dict = None,
                 max_retries: int = 3,
                 ):
        """
        Init lybic client with org_id, api_key and endpoint

        :param org_id:
        :param api_key:
        :param endpoint:
        :param max_retries: maximum number of retries for failed requests
        """
        super().__init__(
            org_id=org_id, api_key=api_key, endpoint=endpoint,
            timeout=timeout, extra_headers=extra_headers, max_retries=max_retries
        )

        self.client: httpx.AsyncClient | None = None
        self._in_context = False

    def _ensure_client_is_open(self):
        if self.client is None:
            self.client = httpx.AsyncClient(headers=self.headers, timeout=self.timeout)
        elif self.client.is_closed:
            raise RuntimeError("The client has been closed and cannot be reused. Please create a new client instance.")

    async def __aenter__(self):
        if self._in_context:
            raise RuntimeError("Cannot re-enter context.")
        if self.client and not self.client.is_closed:
            raise RuntimeError("Cannot enter context with an already-active client.")

        self._in_context = True
        self._ensure_client_is_open()
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        self._in_context = False
        await self.close()

    async def close(self):
        """Close the underlying httpx.AsyncClient."""
        if self._in_context:
            return
        if self.client:
            await self.client.aclose()

    async def request(self, method: str, path: str, **kwargs) -> httpx.Response:
        """
        Make a request to Lybic Restful API

        :param method:
        :param path:
        :param kwargs:
        :return:
        """
        self._ensure_client_is_open()

        url = f"{self.endpoint}{path}"
        headers = self.headers.copy()
        if method.upper() != "POST":
            headers.pop("Content-Type", None)

        last_exception = None
        for attempt in range(self.max_retries + 1):
            try:
                response = await self.client.request(method, url, headers=headers, **kwargs)
                response.raise_for_status()
                return response
            except (httpx.RequestError, httpx.HTTPStatusError) as e:
                last_exception = e
                if attempt < self.max_retries:
                    self.logger.debug(f"Request failed (attempt {attempt + 1}/{self.max_retries + 1}): {e}")
                else:
                    self.logger.error("Request failed after %d attempts", self.max_retries + 1)
                await asyncio.sleep(2 ** attempt)

        raise last_exception



# -*- coding: UTF-8 -*-
#
# Copyright (c) 2019-2025   Beijing Tingyu Technology Co., Ltd.
# Copyright (c) 2025        Lybic Development Team <team@lybic.ai, lybic@tingyutech.com>
# Copyright (c) 2025        Lu Yicheng <luyicheng@tingyutech.com>
#
# Author: AEnjoy <aenjoyable@163.com>
#
# These Terms of Service ("Terms") set forth the rules governing your access to and use of the website lybic.ai
# ("Website"), our web applications, and other services (collectively, the "Services") provided by Beijing Tingyu
# Technology Co., Ltd. ("Company," "we," "us," or "our"), a company registered in Haidian District, Beijing. Any
# breach of these Terms may result in the suspension or termination of your access to the Services.
# By accessing and using the Services and/or the Website, you represent that you are at least 18 years old,
# acknowledge that you have read and understood these Terms, and agree to be bound by them. By using or accessing
# the Services and/or the Website, you further represent and warrant that you have the legal capacity and authority
# to agree to these Terms, whether as an individual or on behalf of a company. If you do not agree to all of these
# Terms, do not access or use the Website or Services.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

"""dto.py provides all the data types used in the API."""
import uuid
from enum import Enum, unique
from typing import List, Optional, Union, Literal
from pydantic import BaseModel, Field, RootModel

from lybic._api import deprecated


class StatsResponseDto(BaseModel):
    """
    Organization Stats response.
    """
    mcpServers: int
    sandboxes: int
    projects: int


class McpServerPolicy(BaseModel):
    """
    MCP server sandbox policy.
    """
    sandboxMaxLifetimeSeconds: int = Field(3600, description="The maximum lifetime of a sandbox.")
    sandboxMaxIdleTimeSeconds: int = Field(3600, description="The maximum idle time of a sandbox.")
    sandboxAutoCreation: bool = Field(False,
                                      description="Whether to create a new sandbox automatically when old sandbox is deleted. If not, new sandboxes will be created when calling computer use tools.")
    sandboxExposeRecreateTool: bool = Field(False, description="Whether to expose recreate tool to LLMs.")
    sandboxExposeRestartTool: bool = Field(False, description="Whether to expose restart tool to LLMs.")
    sandboxExposeDeleteTool: bool = Field(False, description="Whether to expose delete tool to LLMs.")


class McpServerResponseDto(BaseModel):
    """
    MCP server response.
    """
    id: str = Field(..., description="ID of the MCP server.")
    name: str = Field(..., description="Name of the MCP server.")
    createdAt: str = Field(..., description="Creation date of the MCP server.")
    defaultMcpServer: bool = Field(..., description="Whether this is the default MCP server for the organization.")
    projectId: str = Field(..., description="Project ID to which the MCP server belongs.")
    policy: McpServerPolicy
    currentSandboxId: Optional[str] = Field(None, description="ID of the currently connected sandbox.")


class ListMcpServerResponse(RootModel):
    """
    A list of MCP server responses.
    """
    root: List[McpServerResponseDto]

    def __iter__(self):
        return iter(self.root)

    def __getitem__(self, item):
        return self.root[item]


class CreateMcpServerDto(McpServerPolicy):
    """
    Create MCP server request.
    Only name is needed, other fields are optional.
    """
    name: str = Field(..., description="Name of the MCP server.")
    projectId: Optional[str] = Field('', description="Project to which the MCP server belongs to.")

    sandboxMaxLifetimeSeconds: Optional[int] = Field(3600, description="The maximum lifetime of a sandbox.")
    sandboxMaxIdleTimeSeconds: Optional[int] = Field(3600, description="The maximum idle time of a sandbox.")
    sandboxAutoCreation: Optional[bool] = Field(False,
                                                description="Whether to create a new sandbox automatically when old sandbox is deleted. If not, new sandboxes will be created when calling computer use tools.")
    sandboxExposeRecreateTool: Optional[bool] = Field(False, description="Whether to expose recreate tool to LLMs.")
    sandboxExposeRestartTool: Optional[bool] = Field(False, description="Whether to expose restart tool to LLMs.")
    sandboxExposeDeleteTool: Optional[bool] = Field(False, description="Whether to expose delete tool to LLMs.")

    class Config:
        """
        Configuration for Pydantic model.
        """
        extra = "ignore"
        # Allow population of fields with default values
        validate_assignment = True


# Sandbox Schemas
class Sandbox(BaseModel):
    """
    Represents a sandbox environment.
    """
    id: str = Field(..., description="ID of the sandbox.")
    name: str = Field(..., description="Name of the sandbox.")
    expiredAt: str = Field(..., description="Expiration date of the sandbox.")
    createdAt: str = Field(..., description="Creation date of the sandbox.")
    projectId: str = Field(..., description="Project ID to which the sandbox belongs.")


class GatewayAddress(BaseModel):
    """
    Details of a gateway address for connecting to a sandbox.
    """
    address: str
    port: int
    name: str
    preferredProviders: List[Literal["CHINA_MOBILE", "CHINA_UNICOM", "CHINA_TELECOM", "GLOBAL_BGP", 1, 2, 3, 4]]
    gatewayType: Literal["KCP", "QUIC", "WEB_TRANSPORT", 4, 5, 6]


class ConnectDetails(BaseModel):
    """
    Connection details for a sandbox, including gateway addresses and authentication tokens.
    """
    gatewayAddresses: List[GatewayAddress]
    certificateHashBase64: str
    endUserToken: str
    roomId: str


class SandboxListItem(Sandbox):
    """
    An item in a list of sandboxes, containing sandbox details and connection information.
    """


class SandboxListResponseDto(RootModel):
    """
    A response DTO containing a list of sandboxes.
    """
    root: List[SandboxListItem]

    def __iter__(self):
        return iter(self.root)

    def __getitem__(self, item):
        return self.root[item]


class CreateSandboxDto(BaseModel):
    """
    Create sandbox request.
    """
    name: str = Field("sandbox", description="The name of the sandbox.")
    maxLifeSeconds: int = Field(3600,
                                description="The maximum life time of the sandbox in seconds. Default is 1 hour, max is 1 day.",
                                ge=1, le=86400)
    projectId: Optional[str] = Field(None, description="The project id to use for the sandbox. Use default if not provided.")
    specId: Optional[str] = Field(None, description="The spec of the sandbox. Use default if not provided.")
    datacenterId: Optional[str] = Field(None, description="The datacenter id to use for the sandbox. Use default if not provided.")

    class Config:
        """
        Configuration for Pydantic model.
        """
        exclude_none = True


class GetSandboxResponseDto(BaseModel):
    """
    A response DTO for a single sandbox, including connection details.
    """
    sandbox: Sandbox
    connectDetails: ConnectDetails


# Computer Use Schemas
class PixelLength(BaseModel):
    """
    Represents a length in pixels.
    """
    type: Literal["px"]
    value: int


class FractionalLength(BaseModel):
    """
    Represents a length as a fraction of a total dimension.
    """
    type: Literal["/"]
    numerator: int
    denominator: int


Length = Union[PixelLength, FractionalLength]


class MouseClickAction(BaseModel):
    """
    Represents a mouse click action at a specified location.
    """
    type: Literal["mouse:click"]
    x: Length
    y: Length
    button: int = Field(..., description="Mouse button flag combination. 1: left, 2: right, 4: middle, 8: back, 16: forward; add them together to press multiple buttons at once.")
    holdKey: Optional[str] = Field(None, description="Key to hold down during click, in xdotool key syntax. Example: \"ctrl\", \"alt\", \"alt+shift\"")
    callId: Optional[str] = str(uuid.uuid4())

    class Config:
        """
        Configuration for Pydantic model.
        """
        extra = "forbid"
        # Allow population of fields with default values
        validate_assignment = True
        exclude_none = True


class MouseDoubleClickAction(BaseModel):
    """
    Represents a mouse double-click action at a specified location.
    """
    type: Literal["mouse:doubleClick"]
    x: Length
    y: Length
    button: int = Field(..., description="Mouse button flag combination. 1: left, 2: right, 4: middle, 8: back, 16: forward; add them together to press multiple buttons at once.")
    holdKey: Optional[str] = Field(None, description="Key to hold down during click, in xdotool key syntax. Example: \"ctrl\", \"alt\", \"alt+shift\"")
    callId: Optional[str] = str(uuid.uuid4())

    class Config:
        """
        Configuration for Pydantic model.
        """
        extra = "forbid"
        # Allow population of fields with default values
        validate_assignment = True
        exclude_none = True


class MouseMoveAction(BaseModel):
    """
    Represents a mouse move action to a specified location.
    """
    type: Literal["mouse:move"]
    x: Length
    y: Length
    holdKey: Optional[str] = Field(None, description="Key to hold down during click, in xdotool key syntax. Example: \"ctrl\", \"alt\", \"alt+shift\"")
    callId: Optional[str] = str(uuid.uuid4())

    class Config:
        """
        Configuration for Pydantic model.
        """
        extra = "forbid"
        # Allow population of fields with default values
        validate_assignment = True
        exclude_none = True


class MouseScrollAction(BaseModel):
    """
    Represents a mouse scroll action.
    """
    type: Literal["mouse:scroll"]
    x: Length
    y: Length
    stepVertical: int
    stepHorizontal: int
    holdKey: Optional[str] = Field(None, description="Key to hold down during click, in xdotool key syntax. Example: \"ctrl\", \"alt\", \"alt+shift\"")
    callId: Optional[str] = str(uuid.uuid4())

    class Config:
        """
        Configuration for Pydantic model.
        """
        extra = "forbid"
        # Allow population of fields with default values
        validate_assignment = True
        exclude_none = True


class MouseDragAction(BaseModel):
    """
    Represents a mouse drag action from a start to an end point.
    """
    type: Literal["mouse:drag"]
    startX: Length
    startY: Length
    endX: Length
    endY: Length
    holdKey: Optional[str] = Field(None, description="Key to hold down during click, in xdotool key syntax. Example: \"ctrl\", \"alt\", \"alt+shift\"")
    callId: Optional[str] = str(uuid.uuid4())

    class Config:
        """
        Configuration for Pydantic model.
        """
        extra = "forbid"
        # Allow population of fields with default values
        validate_assignment = True
        exclude_none = True


class KeyboardTypeAction(BaseModel):
    """
    Represents a keyboard typing action.
    """
    type: Literal["keyboard:type"]
    content: str
    treatNewLineAsEnter: bool = Field(False, description="Whether to treat line breaks as enter. If true, any line breaks(\\n) in content will be treated as enter key press, and content will be split into multiple lines.")
    callId: Optional[str] = str(uuid.uuid4())


class KeyboardHotkeyAction(BaseModel):
    """
    Represents a keyboard hotkey combination action.
    """
    type: Literal["keyboard:hotkey"]
    keys: str
    duration: Optional[int] = Field(None, description="Duration in milliseconds. If specified, the hotkey will be held for a while and then released.")
    callId: Optional[str] = str(uuid.uuid4())

    class Config:
        """
        Configuration for Pydantic model.
        """
        extra = "forbid"
        # Allow population of fields with default values
        validate_assignment = True
        exclude_none = True


class ScreenshotAction(BaseModel):
    """
    Represents an action to take a screenshot.
    """
    type: Literal["screenshot"]
    callId: Optional[str] = str(uuid.uuid4())

    class Config:
        """
        Configuration for Pydantic model.
        """
        extra = "forbid"
        # Allow population of fields with default values
        validate_assignment = True
        exclude_none = True


class WaitAction(BaseModel):
    """
    Represents a wait action for a specified duration.
    """
    type: Literal["wait"]
    duration: int
    callId: Optional[str] = str(uuid.uuid4())

    class Config:
        """
        Configuration for Pydantic model.
        """
        # Allow population of fields with default values
        validate_assignment = True
        exclude_none = True


class FinishedAction(BaseModel):
    """
    Represents a finished action, signaling successful completion of a task.
    """
    type: Literal["finished"]
    message: Optional[str] = None
    callId: Optional[str] = str(uuid.uuid4())

    class Config:
        """
        Configuration for Pydantic model.
        """
        # Allow population of fields with default values
        validate_assignment = True
        exclude_none = True


class FailedAction(BaseModel):
    """
    Represents a failed action, signaling an error or failure in a task.
    """
    type: Literal["failed"]
    message: Optional[str] = None
    callId: Optional[str] = str(uuid.uuid4())

    class Config:
        """
        Configuration for Pydantic model.
        """
        # Allow population of fields with default values
        validate_assignment = True
        exclude_none = True


ComputerUseAction = Union[
    MouseClickAction,
    MouseDoubleClickAction,
    MouseMoveAction,
    MouseScrollAction,
    MouseDragAction,
    KeyboardTypeAction,
    KeyboardHotkeyAction,
    ScreenshotAction,
    WaitAction,
    FinishedAction,
    FailedAction,
]


class ComputerUseActionDto(BaseModel):
    """
    Computer use action request.
    """
    action: ComputerUseAction | dict
    includeScreenShot: bool = True
    includeCursorPosition: bool = True
    callId: Optional[str] = str(uuid.uuid4())

    class Config:
        """
        Configuration for Pydantic model.
        """
        extra = "forbid"
        # Allow population of fields with default values
        validate_assignment = True
        exclude_none = True


class CursorPosition(BaseModel):
    """
    Represents the position of the cursor on the screen.
    """
    x: int
    y: int
    screenWidth: int
    screenHeight: int
    screenIndex: int


class ExtendSandboxDto(BaseModel):
    """
    Extend sandbox life request.
    """
    maxLifeSeconds: int = Field(3600, description="Max life seconds of sandbox", ge=30, le=60 * 60 * 24)


class SandboxActionResponseDto(BaseModel):
    """
    Computer use action response.
    """
    screenShot: Optional[str]  # is a picture url of the screen eg. https://example.com/screen.webp
    cursorPosition: Optional[CursorPosition]


@unique
class ModelType(Enum):
    """
    Enumeration of supported LLM models for computer-use parsing.
    """
    UITARS = "ui-tars"
    SEED = "seed"
    GLM_4_1V = "glm-4.1v"
    GLM_4_5V = "glm-4.5v"
    QWEN_2_5_VL = "qwen-2.5-vl"


@deprecated(
    since="0.7.0",
    removal="1.0.0",
    message=(
        "Starting from v0.7.0, parsing LLM output functions(ComputerUse.parse_llm_output) will "
        "no longer require ComputerUseParseRequestDto"
    )
)
class ComputerUseParseRequestDto(BaseModel):
    """
    Request DTO for parsing text content into computer use actions.
    """
    model: Literal["ui-tars", "oai-compute-use", "seed"]
    textContent: str


class ComputerUseActionResponseDto(BaseModel):
    """
    Response DTO containing a list of parsed computer use actions.
    """
    unknown: Optional[str] = None
    thoughts: Optional[str] = None
    memory: Optional[str] = None

    actions: List[ComputerUseAction]


# Project Schemas
class ProjectResponseDto(BaseModel):
    """
    Get Project Response
    """
    id: str
    name: str
    createdAt: str
    defaultProject: bool


class ListProjectsResponseDto(RootModel):
    """
    A response DTO containing a list of projects.
    """
    root: List[ProjectResponseDto]

    def __iter__(self):
        return iter(self.root)

    def __getitem__(self, item):
        return self.root[item]


class CreateProjectDto(BaseModel):
    """
    Data transfer object for creating a new project.
    """
    name: str


class SingleProjectResponseDto(ProjectResponseDto):
    """
    Response DTO for a single project.
    """


class SetMcpServerToSandboxResponseDto(BaseModel):
    """
    Response DTO for setting a MCP server to a sandbox.
    """
    sandboxId: Optional[str] = Field(None, description="The ID of the sandbox to connect the MCP server to.")



# -*- coding: UTF-8 -*-
#
# Copyright (c) 2019-2025   Beijing Tingyu Technology Co., Ltd.
# Copyright (c) 2025        Lybic Development Team <team@lybic.ai, lybic@tingyutech.com>
# Copyright (c) 2025        Lu Yicheng <luyicheng@tingyutech.com>
#
# Author: AEnjoy <aenjoyable@163.com>
#
# These Terms of Service ("Terms") set forth the rules governing your access to and use of the website lybic.ai
# ("Website"), our web applications, and other services (collectively, the "Services") provided by Beijing Tingyu
# Technology Co., Ltd. ("Company," "we," "us," or "our"), a company registered in Haidian District, Beijing. Any
# breach of these Terms may result in the suspension or termination of your access to the Services.
# By accessing and using the Services and/or the Website, you represent that you are at least 18 years old,
# acknowledge that you have read and understood these Terms, and agree to be bound by them. By using or accessing
# the Services and/or the Website, you further represent and warrant that you have the legal capacity and authority
# to agree to these Terms, whether as an individual or on behalf of a company. If you do not agree to all of these
# Terms, do not access or use the Website or Services.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

"""base.py holds the base client for Lybic API."""
import logging
import os
from sys import stderr


class _LybicBaseClient:
    """_LybicBaseClient is a base client for all Lybic API."""

    def __init__(self,
                 org_id: str = os.getenv("LYBIC_ORG_ID"),
                 api_key: str = os.getenv("LYBIC_API_KEY"),
                 endpoint: str = os.getenv("LYBIC_API_ENDPOINT", "https://api.lybic.cn"),
                 timeout: int = 10,
                 extra_headers: dict = None,
                 max_retries: int = 3,
                 ):
        """
        Init lybic client with org_id, api_key and endpoint

        :param org_id:
        :param api_key:
        :param endpoint:
        """
        assert org_id, "LYBIC_ORG_ID is required"
        assert endpoint, "LYBIC_API_ENDPOINT is required"

        self.headers = {}
        if extra_headers:
            self.headers.update(extra_headers)

        # if x-trial-session-token is provided, use it instead of api_key
        if not (extra_headers and 'x-trial-session-token' in extra_headers):
            assert api_key, "LYBIC_API_KEY is required when x-trial-session-token is not provided"
            self.headers["x-api-key"] = api_key

        if endpoint.endswith("/"):
            self.endpoint = endpoint[:-1]
        else:
            self.endpoint = endpoint

        if timeout < 0:
            print("Warning: Timeout cannot be negative, set to 10", file=stderr)
            timeout = 10
        self.timeout = timeout
        self.org_id = org_id
        self.max_retries = max(max_retries, 0)
        self.headers["Content-Type"] = "application/json"

        self.logger = logging.getLogger(__name__)

    def make_mcp_endpoint(self, mcp_server_id: str) -> str:
        """
        Make MCP endpoint for a MCP server

        :param mcp_server_id:
        :return:
        """
        return f"{self.endpoint}/mcp/{mcp_server_id}"



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
# Copyright (c) 2019-2025   Beijing Tingyu Technology Co., Ltd.
# Copyright (c) 2025        Lybic Development Team <team@lybic.ai, lybic@tingyutech.com>
# Copyright (c) 2025        Lu Yicheng <luyicheng@tingyutech.com>
#
# Author: AEnjoy <aenjoyable@163.com>
#
# These Terms of Service ("Terms") set forth the rules governing your access to and use of the website lybic.ai
# ("Website"), our web applications, and other services (collectively, the "Services") provided by Beijing Tingyu
# Technology Co., Ltd. ("Company," "we," "us," or "our"), a company registered in Haidian District, Beijing. Any
# breach of these Terms may result in the suspension or termination of your access to the Services.
# By accessing and using the Services and/or the Website, you represent that you are at least 18 years old,
# acknowledge that you have read and understood these Terms, and agree to be bound by them. By using or accessing
# the Services and/or the Website, you further represent and warrant that you have the legal capacity and authority
# to agree to these Terms, whether as an individual or on behalf of a company. If you do not agree to all of these
# Terms, do not access or use the Website or Services.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

"""Setup scripts for build"""

from setuptools import setup, find_packages
with open('README.md', encoding='utf-8') as f:
    setup(
        name='lybic',
        author='Lybic Development Team',
        author_email='team@lybic.ai',
        description='Lybic sdk for python',
        long_description=f.read(),
        long_description_content_type='text/markdown',
        url='https://github.com/lybic/lybic-sdk-python',
        packages=find_packages(),
        classifiers=[
            'Programming Language :: Python :: 3',
            'Operating System :: OS Independent',
            'License :: OSI Approved :: MIT License',
        ],
        license = "MIT",
        python_requires='>=3.12',
        install_requires=[
            'httpx',
            'pydantic',
            'mcp',
            'pillow'
        ],
    )



# -*- coding: UTF-8 -*-
#
# Copyright (c) 2019-2025   Beijing Tingyu Technology Co., Ltd.
# Copyright (c) 2025        Lybic Development Team <team@lybic.ai, lybic@tingyutech.com>
# Copyright (c) 2025        Lu Yicheng <luyicheng@tingyutech.com>
#
# Author: AEnjoy <aenjoyable@163.com>
#
# These Terms of Service ("Terms") set forth the rules governing your access to and use of the website lybic.ai
# ("Website"), our web applications, and other services (collectively, the "Services") provided by Beijing Tingyu
# Technology Co., Ltd. ("Company," "we," "us," or "our"), a company registered in Haidian District, Beijing. Any
# breach of these Terms may result in the suspension or termination of your access to the Services.
# By accessing and using the Services and/or the Website, you represent that you are at least 18 years old,
# acknowledge that you have read and understood these Terms, and agree to be bound by them. By using or accessing
# the Services and/or the Website, you further represent and warrant that you have the legal capacity and authority
# to agree to these Terms, whether as an individual or on behalf of a company. If you do not agree to all of these
# Terms, do not access or use the Website or Services.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

"""
pyautogui.py implements a calling interface compatible with pyautogui.py through lybic

from lybic import LybicClient, Pyautogui

client = LybicClient()
pyautogui = Pyautogui(client, 'your-sandbox-id')

pyautogui.position()
pyautogui.moveTo(1443,343)
pyautogui.click()
pyautogui.click(x=1443, y=343)
pyautogui.move(None, 10)
pyautogui.doubleClick()
pyautogui.moveTo(500, 500)
pyautogui.write('Hello world!')
pyautogui.press('esc')
pyautogui.keyDown('shift')
pyautogui.keyUp('shift')
pyautogui.hotkey('ctrl', 'c')
"""
import asyncio
import logging
import threading
import time
from typing import overload, Optional, Coroutine, List

from lybic import dto
from lybic.lybic import LybicClient
from lybic.mcp import ComputerUse

# pylint: disable=unused-argument,invalid-name,logging-fstring-interpolation
class Pyautogui:
    """
    Pyautogui implements a calling interface compatible with pyautogui.py through lybic

    Examples:

    LLM_OUTPUT = 'pyautogui.click(x=1443, y=343)'

    from lybic import LybicClient, Pyautogui

    client = LybicClient()

    pyautogui = Pyautogui(client,sandbox_id)

    eval(LLM_OUTPUT)
    """
    def __init__(self, client: LybicClient, sandbox_id: str):
        self.client = client
        self.computer_use = ComputerUse(client)
        self.sandbox_id = sandbox_id
        self.logger = logging.getLogger(__name__)

        self._loop = asyncio.new_event_loop()
        self._thread = threading.Thread(target=self._loop.run_forever, daemon=True)
        self._thread.start()
        self.logger.info("PyautoguiLybic event loop running in background thread.")

    def _run_sync(self, coro: Coroutine):
        """Runs a coroutine in the background event loop and waits for the result."""
        future = asyncio.run_coroutine_threadsafe(coro, self._loop)
        return future.result()

    def close(self):
        """Stops the background event loop and thread."""
        if self._thread.is_alive():
            self.logger.info("Closing PyautoguiLybic background event loop.")
            self._loop.call_soon_threadsafe(self._loop.stop)
            self._thread.join()
            self.logger.info("PyautoguiLybic background thread closed.")

    def __del__(self):
        self.close()

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close()

    @overload
    def clone(self, sandbox_id: str) -> "Pyautogui": ...

    @overload
    def clone(self) -> "Pyautogui": ...

    def clone(self, sandbox_id: str = None) -> "Pyautogui":
        """
        Clones the Pyautogui object with a new sandbox ID.

        Args:
            sandbox_id (str, optional): The sandbox ID to clone the object with. If not provided, the original sandbox ID will be used.

        Returns:
            Pyautogui: A new Pyautogui object with the specified sandbox ID.
        Note: The cloned object will have its own background thread. Frequent cloning may lead to high resource consumption.
        """
        if sandbox_id is not None:
            return Pyautogui(self.client, sandbox_id)
        return Pyautogui(self.client, self.sandbox_id)

    def position(self) -> tuple[int, int]:
        """
        Returns the current mouse position.

        Returns:
            tuple[int, int]: The current mouse position as a tuple of (x, y).
        """
        return self.get_mouse_position()

    def get_mouse_position(self) -> tuple[int, int]:
        """
        Returns the current mouse position.

        Returns:
            tuple[int, int]: The current mouse position as a tuple of (x, y).
        """
        coro = self.computer_use.execute_computer_use_action(
            sandbox_id=self.sandbox_id,
            # An action is required to obtain the mouse cursor and screenshot information.
            #
            # The `FinishedAction` , however, does not involve any action operations, is idempotent,
            # and offers the highest performance.
            data=dto.ComputerUseActionDto(
                action=dto.FinishedAction(type="finished"),
                includeScreenShot=False,
                includeCursorPosition=True
            ),
        )
        result = self._run_sync(coro)
        if result.cursorPosition:
            return result.cursorPosition.x, result.cursorPosition.y
        raise ConnectionError("Could not get mouse position")

    def moveTo(self, x, y, duration=0.0, tween=None, logScreenshot=False, _pause=True):
        """
        Moves the mouse to the specified position.

        Args:
            x (int): The x-coordinate of the destination position.
            y (int): The y-coordinate of the destination position.
            duration (Placeholder):
            tween (Placeholder):
            logScreenshot (Placeholder):
            _pause (Placeholder):
        """
        request = dto.MouseMoveAction(
            type="mouse:move",
            x=dto.PixelLength(type="px", value=x),
            y=dto.PixelLength(type="px", value=y),
        )
        coro = self.computer_use.execute_computer_use_action(
            sandbox_id=self.sandbox_id,
            data=dto.ComputerUseActionDto(action=request, includeScreenShot=False, includeCursorPosition=False)
        )
        self._run_sync(coro)

    def move(self, xOffset=None, yOffset=None, duration=0.0, tween=None, _pause=True):
        """
        Moves the mouse relative to its current position.

        Args:
            xOffset (int, optional): The x-coordinate offset. If None, the current x-coordinate will be used.
            yOffset (int, optional): The y-coordinate offset. If None, the current y-coordinate will be used.
            duration (Placeholder):
            tween (Placeholder):
            _pause (Placeholder):
        """
        if xOffset is None and yOffset is None:
            return

        current_x, current_y = self.position()
        xOffset = xOffset if xOffset is not None else 0
        yOffset = yOffset if yOffset is not None else 0

        new_x = current_x + xOffset
        new_y = current_y + yOffset
        self.moveTo(new_x, new_y, duration, tween, _pause=_pause)

    def click(self, x: Optional[int] = None, y: Optional[int] = None,
              clicks=1, interval=0.0, button='left', duration=0.0, tween=None,
              logScreenshot=None, _pause=True):
        """
        Performs a mouse click at the specified position.

        Args:
            x (int, optional): The x-coordinate of the click position. If None, the current mouse position will be used.
            y (int, optional): The y-coordinate of the click position. If None, the current mouse position will be used.
            clicks (int, optional): The number of clicks to perform. Defaults to 1.
            interval (Placeholder):
            button (str, optional): The button to click. Can be 'left', 'right', or 'middle'. Defaults to 'left'.
        """
        if x is None or y is None:
            x, y = self.position()

        self.logger.info(f"click(x={x}, y={y}, clicks={clicks}, button='{button}')")

        button_map = {'left': 1, 'right': 2, 'middle': 4}
        button_code = button_map.get(button.lower(), 1)

        if clicks == 2:
            action = dto.MouseDoubleClickAction(
                type="mouse:doubleClick",
                x=dto.PixelLength(type="px", value=x),
                y=dto.PixelLength(type="px", value=y),
                button=button_code
            )
            coro = self.computer_use.execute_computer_use_action(
                sandbox_id=self.sandbox_id,
                data=dto.ComputerUseActionDto(action=action, includeScreenShot=False,
                                              includeCursorPosition=False)
            )
            self._run_sync(coro)
        else:
            for i in range(clicks):
                action = dto.MouseClickAction(
                    type="mouse:click",
                    x=dto.PixelLength(type="px", value=x),
                    y=dto.PixelLength(type="px", value=y),
                    button=button_code
                )

                coro = self.computer_use.execute_computer_use_action(
                    sandbox_id=self.sandbox_id,
                    data=dto.ComputerUseActionDto(action=action, includeScreenShot=False,
                                                  includeCursorPosition=False)
                )
                self._run_sync(coro)

                if i < clicks - 1:
                    time.sleep(interval)

    def doubleClick(self, x: Optional[int] = None, y: Optional[int] = None,
                    interval=0.0, button='left', duration=0.0, tween=None, _pause=True):
        """
        Performs a double-click at the specified position.

        Args:
            x (int, optional): The x-coordinate of the click position. If None, the current mouse position will be used.
            y (int, optional): The y-coordinate of the click position. If None, the current mouse position will be used.
            interval (Placeholder):
            button (str, optional): The button to click. Can be 'left', 'right', or 'middle'. Defaults to 'left'.
        """
        self.click(x, y, clicks=2, interval=interval, button=button, duration=duration, tween=tween, _pause=_pause)

    def write(self, message, interval=0.0, _pause=True):
        """
        Types the specified message into the keyboard.

        Args:
            message (str): The message to type.
            interval (Placeholder):
        """
        request = dto.KeyboardTypeAction(
            type="keyboard:type",
            content=message
        )
        coro = self.computer_use.execute_computer_use_action(
            sandbox_id=self.sandbox_id,
            data=dto.ComputerUseActionDto(action=request, includeScreenShot=False, includeCursorPosition=False)
        )
        self._run_sync(coro)

    @overload
    def press(self, keys: str, presses: int = 1, interval: float = 0.0, _pause: bool = True): ...

    @overload
    def press(self, keys: List[str], presses: int = 1, interval: float = 0.0, _pause: bool = True): ...

    def press(self, keys, presses=1, interval=0.0, _pause=True):
        """
        Presses the specified keys.

        Args:
            keys (str or List[str]): The key to press, or a list of keys to press in sequence.
            presses (int, optional): The number of times to press the keys. Defaults to 1.
            interval (float, optional): The interval in seconds between each press. Defaults to 0.0
        """
        if isinstance(keys, str):
            _keys = [keys] * presses
        else:
            _keys = keys * presses

        for i, key in enumerate(_keys):
            request = dto.KeyboardHotkeyAction(
                type="keyboard:hotkey",
                keys=key
            )
            coro = self.computer_use.execute_computer_use_action(
                sandbox_id=self.sandbox_id,
                data=dto.ComputerUseActionDto(action=request, includeScreenShot=False,
                                              includeCursorPosition=False)
            )
            self._run_sync(coro)
            if i < len(_keys) - 1:
                time.sleep(interval)

    def hotkey(self, *args, interval=0.0, _pause=True):
        """
        Presses a hotkey combination.

        Args:
            *args (str): The keys to press.
            interval (Placeholder):
        """
        keys_to_press = '+'.join(args)
        request = dto.KeyboardHotkeyAction(
            type="keyboard:hotkey",
            keys=keys_to_press
        )
        coro = self.computer_use.execute_computer_use_action(
            sandbox_id=self.sandbox_id,
            data=dto.ComputerUseActionDto(action=request, includeScreenShot=False, includeCursorPosition=False)
        )
        self._run_sync(coro)

    def keyDown(self, key):
        """
        Holds down a key. (not implemented)

        Args:
            key (str): The key to hold down.
        """
        self.logger.warning("keyDown is not implemented in Lybic SDK")
        raise NotImplementedError("Lybic API does not support holding a key down.")

    def keyUp(self, key):
        """
        Releases a key. (not implemented)

        Args:
            key (str): The key to release.
        """
        self.logger.warning("keyUp is not implemented in Lybic SDK")
        raise NotImplementedError("Lybic API does not support releasing a key.")



#!/usr/bin/env python3
# -*- coding: UTF-8 -*-
#
# Copyright (c) 2019-2025   Beijing Tingyu Technology Co., Ltd.
# Copyright (c) 2025        Lybic Development Team <team@lybic.ai, lybic@tingyutech.com>
# Copyright (c) 2025        Lu Yicheng <luyicheng@tingyutech.com>
#
# Author: AEnjoy <aenjoyable@163.com>
#
# These Terms of Service ("Terms") set forth the rules governing your access to and use of the website lybic.ai
# ("Website"), our web applications, and other services (collectively, the "Services") provided by Beijing Tingyu
# Technology Co., Ltd. ("Company," "we," "us," or "our"), a company registered in Haidian District, Beijing. Any
# breach of these Terms may result in the suspension or termination of your access to the Services.
# By accessing and using the Services and/or the Website, you represent that you are at least 18 years old,
# acknowledge that you have read and understood these Terms, and agree to be bound by them. By using or accessing
# the Services and/or the Website, you further represent and warrant that you have the legal capacity and authority
# to agree to these Terms, whether as an individual or on behalf of a company. If you do not agree to all of these
# Terms, do not access or use the Website or Services.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
"""This file is lybic SDK E2E Test scripts"""
import asyncio
from lybic import (Project,
                   MCP,
                   Sandbox,
                   ComputerUse,
                   LybicClient,
                   Stats, Pyautogui)

async def test_stats(client:LybicClient):
    """
    Test Restful API Class: Stats
    :param client:
    :return:
    """
    stats = Stats(client)
    result =  await stats.get()
    print(result)

async def test_project(client:LybicClient):
    """
    Test Restful API Class: Project
    :param client:
    :return:
    """
    project = Project(client)
    print("Test List Project:",await project.list())
    print("Test Create Project:",)
    result = await project.create(name='test_project')
    print("Result:",result)
    print("Test Delete Project")
    await project.delete(result.id)

async def test_sandbox(client:LybicClient):
    """
    Test Restful API Class: Sandbox
    :param client:
    :return:
    """
    await Project(client).create(name='test_sandbox')
    sandbox = Sandbox(client)

    print("Test List Sandbox:",await sandbox.list())
    print("Test Create Sandbox:",await sandbox.create(name="test_sandbox"))

    print("Test Get Sandbox:")
    result = await sandbox.get("test_sandbox")
    print(result)

    print("Test Delete Sandbox",await sandbox.delete(result.sandbox.id))
    print("Test Get Sandbox preview info",await sandbox.preview(result.sandbox.id))
    print("Test Extend Sandbox",await sandbox.extend_life(result.sandbox.id))
    print("Test Get Sandbox connection details",await sandbox.get_connection_details(result.sandbox.id))
    # print("Test Get Sandbox screenshot",await sandbox.get_screenshot(result.sandbox.id))

async def test_mcp(client:LybicClient):
    """
    Test Restful API Class: MCP
    :param client:
    :return:
    """
    await Sandbox(client).create(name='test_mcp')
    mcp = MCP(client)

    print("Test List MCP:",await mcp.list())
    print("Test Create MCP:",await mcp.create(name="test_mcp"))
    print("Test Delete MCP",await mcp.delete("test_mcp"))
    print("Test Set MCP Sandbox",await mcp.set_sandbox("test_mcp","test_sandbox"))
    # print("Test Call MCP Tool",await mcp.call_tool_async("test_mcp"))

async def test_computer_use(client:LybicClient):
    """
    Test Restful API Class: ComputerUse
    :param client:
    :return:
    """
    await Sandbox(client).create(name='test_computer_use')
    computer_use = ComputerUse(client)

    print("Test parse model output:")
    action = await computer_use.parse_model_output(
        model="seed",
        textContent="""Thought: The user wants to open the Chrome app. The screenshot shows the Google Chrome icon on the desktop. I should double-click the Google Chrome icon to open the app.
    Action: left_double(point='<point>10 10</point>')"""
    )
    print("ActionResult:",action)

    sandbox = Sandbox(client)
    print("Execute computer use action:",await sandbox.execute_computer_use_action(
        sandbox_id='test_computer_use',
        action=action.actions[0]
    ))


# pylint: disable=eval-used,fixme
def test_pyautogui(client:LybicClient):
    """
    Test Pyautogui
    :param client:
    :return:
    """
    pyautogui = Pyautogui(client, sandbox_id='test_pyautogui')

    print("Test pyautogui.click without position arguments.")
    pyautogui.click()
    print("Test pyautogui.click with position arguments.")
    pyautogui.click(10, 10)
    print("Test pyautogui.moveTo.")
    pyautogui.moveTo(10, 10)
    print("Test pyautogui.move.")
    pyautogui.move(10, 10)
    print("Test pyautogui.press.")
    pyautogui.press('1')
    print("Test pyautogui.write.")
    pyautogui.write('Hello world!')
    print("Test pyautogui.hotkey.")
    pyautogui.hotkey('ctrl', 'c')

    print("Test Pyautogui Expression Execution")
    expression = 'pyautogui.click(x=1443, y=343)'
    eval(expression)

    pyautogui.close()


async def restful_test() -> None:
    """
    Core restful api test
    """
    async with LybicClient() as client:
        tasks = [
            asyncio.create_task(test_stats(client)),
            asyncio.create_task(test_project(client)),
            asyncio.create_task(test_sandbox(client)),
            asyncio.create_task(test_mcp(client)),
            asyncio.create_task(test_computer_use(client)),
            asyncio.create_task(Sandbox(client).create(name='test_pyautogui'))
        ]
        await asyncio.gather(*tasks)

if __name__ == "__main__":
    # test restful api(asynchronize_test)
    asyncio.run(restful_test())
    # test pyautogui(synchronize_test)
    test_pyautogui(LybicClient())
